; Based on the LockTones clone by Eric Auer.
; HIGHTIME increases int 1c frequency a lot.

; assemble with: nasm hightime.asm -o hightime.com

; Use with care! Original int 8 is called at original
; frequency, at least. You can disable this temporarily
; by setting the byte between >> << to 0. Current setup
; is to increase frequency to 32 * 18.2 Hz (< 1 kHz)

; Function: Run this. Then, int 1ch frequency will
; become pretty high. For threads that want to be waken
; up very often. Let us hope that they read 40:[6c]
; instead of counting how often they are called!!

; Please use nasm to assemble - http://nasm.2y.net/
; nasm is free open source:
; http://www.gnu.org/licenses/licenses.html#LGPL

; This is distributed unter the GPL license,
; http://www.gnu.org/licenses/licenses.html#GPL
; which means it is also free open source.
; Copyright 2003 by Eric Auer, eric (#) coli.uni-sb.de

	org  100h

entry:  jmp near installer

vec8    dw 0,0   ; offset and segment of old handler

	db ">>"
enable  db 1    ; 0 to turn the handler "off"
	db "<<" ; (use normal int 1c freq again)

count   dw 0    ; for frequency division

magic   db 0x0de, 0x0ad, 0x0be, 0x0ef

;------------------


int8:   push ax                 ; our own int 8 / IRQ 0 / timer handler
	inc word [cs:count]     ; our high speed counter
	test word [cs:count],31 ; modulo-style operation
	jz real                 ; every 32th call is for real

faster: cmp byte [cs:enable],1
	jz slower               ; no faster calls if not enabled
	
	int 0x1c                ; FASTER call to int 1c

slower:                         ; do not call any other int
	mov al,0x20             ; consume the (extra) IRQ
	out 0x20,al             ; send EOI to 8259 chip
	pop ax
	iret

real:   pop ax                  ; real int 8 also calls int 1c
	jmp far [cs:vec8]       ; chain to real int 8 handler


; ------------------------------

installer:
	mov dx,thelastbyte      ; message offset
	mov ax,cs               ; message segment
	mov ds,ax
	mov ah,9        ; string$ output
	int 0x21        ; say hello

	mov ax,0x3508   ; remember timer int vector
	int 0x21
	mov [vec8+2],es
	mov [vec8],bx
	cmp bx,int8
	jnz mortal
	mov ax,[es:magic]
	cmp ax,[cs:magic]       ; already another instance resident?
	jnz mortal              ; no, not magic, mere mortal
	mov ax,[es:magic+2]
	cmp ax,[cs:magic+2]
	jnz mortal

duplicate:                      ; too much magic - already installed
	mov dx,duplmsg
	mov ah,9        ; string$ output
	int 0x21        ; say goodbye
	mov ax,0x4c01   ; exit with errorlevel 1
	int 0x21

mortal: mov dx,int8     ; the new handler offset
	mov ax,cs       ; the new handler segment
	mov ds,ax       
	mov ax,0x2508   ; hook int
	int 0x21

	mov al,0x36     ; reprogram timer 0, IRQ 0.
	out 0x43,al     ; command port
	in al,0x80      ; wait
	mov ax,0x800
	out 0x40,al     ; low part divisor of 1.193 MHz
	in al,0x80      ; wait
	mov al,ah
	out 0x40,al     ; high part divisor
			; divisors are sent to 0x40+timernumber!
	; we speed up * 32: normal divisor is 0xffff, not 0x0800
	
	mov dx,tsrmsg   ; say "I stay"
	mov ah,9        ; string$ output
	int 0x21

	mov dx,thelastbyte+15
	mov cl,4
	shr dx,cl

	mov ax,0x3100   ; go TSR
	int 21h

; ------------------------------

thelastbyte     db "HighTime by Eric Auer "
		db "eric (#) coli.uni-sb.de Nov 2003"
		db 13,10
		db "Int 1c freq now 32 * default!"
		db " (Int 8 freq default.)",13,10,"$"

duplmsg         db "Already resident, not loading again."
		db 13,10,"$"
tsrmsg          db "Going resident!",13,10,"$"
